/****************************************************************************
**  BrowserTickTock - Timer/Clock/Alarm add-on for Mozilla FireFox
**  Copyright (c) 2004-2005  Charles Melhorn
**
**  This file is part of BrowserTickTock.
**
**  BrowserTickTock is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  BrowserTickTock is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 


const gBTTconstants =
{
  AS_OFF:          0,  // alarm states
  AS_SET:          1,
  AS_ALARMING:     2,

  AT_TIMER:        0,  // alarm types
  AT_ALARM:        1,
  AT_PERIODIC:     2,

  TP_QUARTERHR:    15, // target periods
  TP_HALFHR:       30,
  TP_HOURLY:       60,

  CF_12HR:         0,  // clock formats
  CF_24HR:         1,

  CI_HOUR:         0,  // cuckoo interval
  CI_HALFHOUR:     1,

  DT_TIME:         0,  // time display type
  DT_DATE:         1,
  DT_REMAIN:       2,
  DT_UPTIME:       3
}

// Date formatting rules support a subset of the format
// characters for JSLib date(), which itself is modeled on
// the PHP date() function. Rules (regular expressions) are
// taken from JSLib date.js, minus the g flag to improve performance.
  
var gdateFormatRules = [ {rule:  /([^\\])d|^d/,
                          maker: function(aDate) {return aDate.getDate();}},

                         {rule:  /([^\\])m|^m/,
                          maker: function(aDate) {return aDate.getMonth() + 1;}},

                         {rule:  /([^\\])Y|^Y/,
                          maker: function(aDate) {return aDate.getFullYear();}},

                         {rule:  /\\([A-Za-z])/g,
                          maker: function(){return ""}}];



var gBTTClockController = 
{
  timeDisplayType:   gBTTconstants.DT_TIME,

  bAlarmSet:         false,
  alarmState:        gBTTconstants.AS_OFF,
  alarm_type:        gBTTconstants.AT_TIMER,
  targetDate:        null,
  origTargetDate:    null,
  countdown_time:    0,
  target_period:     gBTTconstants.TP_HOURLY,
  period_offset:     0,
  snooze_increment:  1,
  auto_reset:        false,
  reminder_msg:      "",

  bCuckooOn:         false,

  sleep_timer:       false
}


const gClockColorPrefListener =
{
  domains: ["extensions.btt.useclockcolor", "extensions.btt.clocktextcolor"],
  observe: function(subject, topic, aPrefString)
  {
    // Is this a pref change?
    if (topic != "nsPref:changed")
      return;

    if (aPrefString == this.domains[0])
     {
       if (!gBTTUserPreferenceInfo.getPrefValue("use_clock_color"))
        {
          document.getElementById("btt-menubarclock-lbl").style.color = "";
        }
      }

    else if (aPrefString == this.domains[1])
     {
        if (gBTTUserPreferenceInfo.getPrefValue("use_clock_color"))
        {
          var clocktext_color = gBTTUserPreferenceInfo.getPrefValue("clocktext_color");
          document.getElementById("btt-menubarclock-lbl").style.color = clocktext_color;

          // hack to force FF to update all instances with new color
          document.getElementById("btt-menubarclock-lbl").style.color = "";
          document.getElementById("btt-menubarclock-lbl").style.color = clocktext_color;
        }
     }
  }
};

var gDbtUiElements = 
{
  menubarclock_lbl:   null,
  s_bundle:           null
}


var gCuckoo_played = false;
var gAlarmcycle_cnt = 0;

var gClock_TimerID = null;
var gAlarm_TimerID = null;

var gBTTUptime;
//var gBTTFirstTime = true;

bttAddPrefListener(gClockColorPrefListener);
//setTimeout("initBrowserTickTock()", 800);
//var update_interval = gBTTUserPreferenceInfo.getPrefValue("update_interval") * 1000;
//var gClock_TimerID = setInterval( "checkTime()", update_interval );



function initBrowserTickTock()
{
   gDbtUiElements.s_bundle = document.getElementById("browserticktock_bundle");
   gDbtUiElements.menubarclock_lbl = document.getElementById("btt-menubarclock-lbl");

   if (gDbtUiElements.menubarclock_lbl)  // check that BTT is on toolbar
    {
      if (gBTTUserPreferenceInfo.getPrefValue("use_clock_color"))
       {
         var clocktext_color = gBTTUserPreferenceInfo.getPrefValue("clocktext_color");
         gDbtUiElements.menubarclock_lbl.style.color = clocktext_color;

         // hack to force FF to redraw with new color
         document.getElementById("btt-menubarclock-lbl").style.color = "";
         document.getElementById("btt-menubarclock-lbl").style.color = clocktext_color;
       }

      var currenttime = new Date();
      gBTTUptime = currenttime; 
      gDbtUiElements.menubarclock_lbl.value = dateToClock(currenttime);
   
      var update_interval = gBTTUserPreferenceInfo.getPrefValue("update_interval") * 1000;
      var gClock_TimerID = setInterval( "checkTime()", update_interval );
    }

}


function dateToClock(theDate)
{
   var formatpref = gBTTUserPreferenceInfo.getPrefValue("clock_format");

   var hour = theDate.getHours();
   var minute = theDate.getMinutes();
   var temp = "";

   if ( formatpref == gBTTconstants.CF_12HR)
    {
      temp += ((hour > 12) ? hour - 12 : hour);

      if (hour == 0)
         temp = "12";
      temp += ((minute < 10) ? ":0" : ":") + minute;
      temp += (hour >= 12) ? " PM" : " AM";
    }
   else
      temp += ((hour < 10) ? "0" : "") + hour + ((minute < 10) ? ":0" : ":") + minute;


   return temp;
}


function shortDate(currentDate)
{
//  var temp = "" + (currentDate.getMonth() + 1) + "/"
//             + currentDate.getDate() + "/" + currentDate.getFullYear();
//
//  return temp;

    var dateString = gBTTUserPreferenceInfo.getPrefValue("date_format");

    for (var i=0; i < gdateFormatRules.length; i++)
     {
       dateString = dateString.replace(gdateFormatRules[i].rule,
                                      "$1" + gdateFormatRules[i].maker(currentDate));
     }
    return dateString;
}


function calculateTimeRemaining(currenttime)
{
  var time_remaining_secs = Math.floor((gBTTClockController.targetDate.getTime() - currenttime.getTime()) / 1000);
  var total_remaining_mins = Math.ceil(time_remaining_secs / 60);
  var time_remaining_hrs = Math.floor(total_remaining_mins / 60);
  var time_remaining_mins = total_remaining_mins % 60;

  var temp = "" + ((time_remaining_hrs < 10) ? "0" : "") + time_remaining_hrs;
  temp += ((time_remaining_mins < 10) ? ":0" : ":") + time_remaining_mins;

  return temp;
}


function calculateBrowserUptime(currenttime)
{
  var uptime_secs = Math.floor((currenttime.getTime() - gBTTUptime.getTime()) / 1000);
  var total_uptime_mins = Math.floor(uptime_secs / 60);
  var uptime_hrs = Math.floor(total_uptime_mins / 60);
  var uptime_mins = total_uptime_mins % 60;

  var temp = "" + ((uptime_hrs < 10) ? "0" : "") + uptime_hrs;
  temp += ((uptime_mins < 10) ? ":0" : ":") + uptime_mins;

  return temp;
}


function checkTime()
{
  var currenttime = new Date();
 
  switch (gBTTClockController.timeDisplayType)
  {
    case gBTTconstants.DT_TIME:
      gDbtUiElements.menubarclock_lbl.value = dateToClock(currenttime);
      break;
    case gBTTconstants.DT_DATE:
      gDbtUiElements.menubarclock_lbl.value = shortDate(currenttime);
      break;
    case gBTTconstants.DT_REMAIN:
      if (gBTTClockController.alarmState == gBTTconstants.AS_SET)
        gDbtUiElements.menubarclock_lbl.value = calculateTimeRemaining(currenttime);
      else
        gDbtUiElements.menubarclock_lbl.value = "00:00";
      break;
    case gBTTconstants.DT_UPTIME:
      gDbtUiElements.menubarclock_lbl.value = calculateBrowserUptime(currenttime);
      break;
    default:
      break;
  }

  if (gBTTClockController.alarmState == gBTTconstants.AS_SET)
   {
     var c_time_in_secs = Math.floor(currenttime.getTime() / 1000);
     var t_time_in_secs = Math.floor(gBTTClockController.targetDate.getTime() / 1000);

     if (c_time_in_secs >= t_time_in_secs)
      {
        gBTTClockController.alarmState = gBTTconstants.AS_ALARMING;
        document.getElementById("btt-alarmicon-img").setAttribute("alarming", "true");
        document.getElementById("btt-alarmicon-img").setAttribute("tooltiptext", "click to snooze");

        if (gBTTUserPreferenceInfo.getPrefValue("alarm_playsound"))
          playAlarmSound();

        gAlarmcycle_cnt = 1;
        gAlarm_TimerID = setInterval( "signalAlarm()", 5000 );

        if (gBTTUserPreferenceInfo.getPrefValue("alarm_flashbrowser") == true)
          window.getAttention();  // flashes window titlebar and taskbar icon if 
                                  // window doesn't have focus

        if (gBTTUserPreferenceInfo.getPrefValue("alarm_showslider") == true)
         { 
           // may not work for some platforms
           try {
             var alertService = Components.classes["@mozilla.org/alerts-service;1"]
                                .getService(Components.interfaces.nsIAlertsService);

             alertService.showAlertNotification("chrome://browserticktock/skin/btt_digital.gif",
                                                "BTT Alarm", gBTTClockController.reminder_msg,
                                                false,      // alert text not clickable
                                                "",         // no cookie
                                                null);      // no callback listener
           } catch (e) {}
         }

        if (gBTTUserPreferenceInfo.getPrefValue("alarm_loadurl") == true)
          bttLoadOneOrMoreURIs(gBTTUserPreferenceInfo.getPrefValue("alarm_url"));

        if (gBTTUserPreferenceInfo.getPrefValue("alarm_popwindow") == true)
         {
          window.openDialog("chrome://browserticktock/content/dialogAlarmNotify.xul",
                            "bttAlarmNotify","chrome,centerscreen,modal", gBTTClockController.reminder_msg);
         }
      }

   }

  // alarm takes precedence over cuckoo function, so don't bother
  // to check whether to cuckoo if alarm is firing

  if ( (gBTTClockController.bCuckooOn) &&
       (gBTTClockController.alarmState != gBTTconstants.AS_ALARMING) )
    {
      var raw_mins = currenttime.getMinutes();
      var mins = raw_mins;

      if (gBTTUserPreferenceInfo.getPrefValue("cuckoo_interval") == gBTTconstants.CI_HALFHOUR)
        mins = mins % 30;

      if ((mins == 0) && (gCuckoo_played == false))
       { 
         var hours = currenttime.getHours() % 12;
         hours = ((hours == 0) ? 12 : hours);
         var chimes = ((raw_mins == 30) ? 1 : hours);
         document.getElementById('btt-cuckooicon-img').setAttribute("alarming", "true");
         SignalCuckoo(chimes);
         gCuckoo_played = true;
       }
      else if ((mins != 0) && (gCuckoo_played == true))
         gCuckoo_played = false;
    }

}


function playAlarmSound()
{
  var alarmsoundtype = gBTTUserPreferenceInfo.getPrefValue("alarm_sound_type");
  var alarmsoundfile = gBTTUserPreferenceInfo.getPrefValue("alarm_sound_file");

  if (alarmsoundtype == "standard")
   {
     playSoundURL("chrome://browserticktock/content/" + alarmsoundfile);
   }
  else if (alarmsoundtype == "custom")
   {
     playSoundURL(alarmsoundfile);
   }
}


function playSoundURL(sURL_str)
{
  var sample = Components.classes["@mozilla.org/sound;1"].createInstance();
  sample = sample.QueryInterface(Components.interfaces.nsISound);
  var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                  .getService(Components.interfaces.nsIIOService);
  var url = ioService.newURI(sURL_str, null, null);
  sample.play(url);

}


function ResetAlarm()
{
  if (gBTTClockController.alarm_type == gBTTconstants.AT_TIMER)
   {
     gBTTClockController.targetDate.setTime(gBTTClockController.targetDate.getTime() +
                                            gBTTClockController.countdown_time * 60 * 1000);
   }
  else if (gBTTClockController.alarm_type == gBTTconstants.AT_ALARM)
   {
     // targetDate might have been changed by snoozing, so
     // use original target date
     gBTTClockController.targetDate.setTime(gBTTClockController.origTargetDate.getTime() + 24*60*60*1000);
     gBTTClockController.origTargetDate.setTime(gBTTClockController.targetDate.getTime());
   }
  else  // gBTTClockController.alarm_type == gBTTconstants.AT_PERIODIC)
   {
     var targetdate = new Date();
     var current_minutes = targetdate.getMinutes();
     //alert("curr minutes: " + current_minutes);
     targetdate.setSeconds(0);
     targetdate.setMilliseconds(0);

     var mins_to_next_period = (gBTTClockController.target_period -
                               (current_minutes % gBTTClockController.target_period));

     var adjusted_for_offset_mins = (mins_to_next_period + gBTTClockController.period_offset)
                                     % gBTTClockController.target_period;

     if (adjusted_for_offset_mins == 0)
       adjusted_for_offset_mins = gBTTClockController.target_period;

     var target_adjust_ms = adjusted_for_offset_mins*60*1000;

     gBTTClockController.targetDate.setTime(targetdate.getTime() + target_adjust_ms);
   }

}


function signalAlarm()
{ 
  if (gAlarmcycle_cnt++ < gBTTUserPreferenceInfo.getPrefValue("alarm_timeout"))
   {
     if (gBTTUserPreferenceInfo.getPrefValue("alarm_playsound"))
       playAlarmSound();
   }

  else // alarm has timed out
   {
     if (gBTTUserPreferenceInfo.getPrefValue("auto_reset") == true)
      {
        ResetAlarm();

        gBTTClockController.alarmState = gBTTconstants.AS_SET;
        document.getElementById("btt-alarmicon-img").setAttribute("alarming", "false");
        var alarmtime_str = gDbtUiElements.s_bundle.getString("btt.tt.alarmtime");
        alarmtime_str += " " + dateToClock(gBTTClockController.targetDate);
        document.getElementById("btt-alarmicon-img").setAttribute("tooltiptext", alarmtime_str);

        if (gAlarm_TimerID != null)
          clearInterval(gAlarm_TimerID);
      }
     else
      {
        bttToggleAlarm("true");
      }
   }
}


/*****************************************************************
*  SignalCuckoo - called when cuckoo is first triggered (on hour *
*           and/or half hour depending on user preference), then *
*           calls itself recursively, pausing each time, until   *
*           the appropriate number of chimes have been sounded.  *
*           When no more chimes remain, it pauses to let the     *
*           last chime finish, then resets the cuckoo indicator  *
*           icon.                                                *
*                                                                *
*****************************************************************/

function SignalCuckoo(chimes_remaining)
{ 
  if (gBTTUserPreferenceInfo.getPrefValue("cuckoo_playsound"))
    playCuckooSound();

  chimes_remaining--;

  if (chimes_remaining > 0)
   {
     setTimeout(SignalCuckoo, 750, chimes_remaining);
   }
  else
   {
     setTimeout("(document.getElementById('btt-cuckooicon-img')).setAttribute('alarming', 'false');", 1000);
   }
}


function playCuckooSound()
{
  var cuckoosoundtype = gBTTUserPreferenceInfo.getPrefValue("cuckoo_sound_type");
  var cuckoosoundfile = gBTTUserPreferenceInfo.getPrefValue("cuckoo_sound_file");

  if (cuckoosoundtype == "standard")
   {
     playSoundURL("chrome://browserticktock/content/" + cuckoosoundfile);
   }
  else if (cuckoosoundtype == "custom")
   {
     playSoundURL(cuckoosoundfile);
   }
}


function bttEnableCuckoo(bEnable_str)
{
  if (bEnable_str == "true")
   {
     gBTTClockController.bCuckooOn = true;
     document.getElementById("btt-cuckooicon-img").setAttribute("disabled", "false");
   }
  else
   {
     gBTTClockController.bCuckooOn = false;
     document.getElementById("btt-cuckooicon-img").setAttribute("disabled", "true");
   }
}


function bttToggleAlarm(bEnable_str)
{
  if (bEnable_str == "false")
   {
     var alarmSettings = 
     {
       bAlarmSet:         false,
       alarm_type:        gBTTconstants.AT_TIMER,
       targetDate:        null,
       origTargetDate:    null,
       countdown_time:    0,
       target_period:     gBTTconstants.TP_HOURLY,
       period_offset:     0,
       auto_reset:        false,
       snooze_increment:  0,
       reminder_msg:      ""
     }

     window.openDialog("chrome://browserticktock/content/dialogAlarmSet.xul",
                       "bttAlarmSet","chrome,modal,centerscreen", alarmSettings);

     if (alarmSettings.bAlarmSet)
      {
        gBTTClockController.alarmState = gBTTconstants.AS_SET;
        gBTTClockController.alarm_type = alarmSettings.alarm_type;
        gBTTClockController.targetDate = alarmSettings.targetDate;
        gBTTClockController.origTargetDate = alarmSettings.origTargetDate;
        gBTTClockController.countdown_time = alarmSettings.countdown_time;
        gBTTClockController.target_period = alarmSettings.target_period;
        gBTTClockController.period_offset = alarmSettings.period_offset;
        gBTTClockController.reminder_msg = alarmSettings.reminder_msg;
        gBTTClockController.snooze_increment = alarmSettings.snooze_increment;

        if (gBTTUserPreferenceInfo.getPrefValue("use_fixed_snooze"))
          gBTTClockController.snooze_increment = gBTTUserPreferenceInfo.getPrefValue("snooze_increment");

        document.getElementById("btt-alarmicon-img").setAttribute("disabled", "false");
        document.getElementById("btt-alarm-mitem").setAttribute("checked", "true");

        var alarmtime_str = gDbtUiElements.s_bundle.getString("btt.tt.alarmtime"); 
        alarmtime_str += " " + dateToClock(gBTTClockController.targetDate);
        document.getElementById("btt-alarmicon-img").setAttribute("tooltiptext", alarmtime_str);
      }
   }
  else
   {
     gBTTClockController.alarmState = gBTTconstants.AS_OFF;
     document.getElementById("btt-alarmicon-img").setAttribute("disabled", "true");
     document.getElementById("btt-alarmicon-img").removeAttribute("tooltiptext");
     document.getElementById("btt-alarmicon-img").setAttribute("alarming", "false");
     document.getElementById("btt-alarm-mitem").setAttribute("checked", "false");

     if (gAlarm_TimerID != null)
       clearInterval(gAlarm_TimerID);
   }
}


function toggleTimeDisplay()
{
  gBTTClockController.timeDisplayType = (gBTTClockController.timeDisplayType + 1) % 4;

  // skip browser uptime if not enabled
  if ( (gBTTClockController.timeDisplayType == gBTTconstants.DT_UPTIME) &&
       (!gBTTUserPreferenceInfo.getPrefValue("compute_uptime")) )
    gBTTClockController.timeDisplayType = (gBTTClockController.timeDisplayType + 1) % 4;

  var currenttime = new Date();
  var clock_display = document.getElementById("btt-menubarclock-lbl");

  switch (gBTTClockController.timeDisplayType)
  {
    case gBTTconstants.DT_TIME:
      clock_display.value = dateToClock(currenttime);
      clock_display.setAttribute("tooltiptext", gDbtUiElements.s_bundle.getString("btt.tt.time"));
      break;
    case gBTTconstants.DT_DATE:
      clock_display.value = shortDate(currenttime);
      clock_display.setAttribute("tooltiptext", gDbtUiElements.s_bundle.getString("btt.tt.date"));
      break;
    case gBTTconstants.DT_REMAIN:
      if (gBTTClockController.alarmState == gBTTconstants.AS_SET)
        clock_display.value = calculateTimeRemaining(currenttime);
      else
        clock_display.value = "00:00";
      clock_display.setAttribute("tooltiptext", gDbtUiElements.s_bundle.getString("btt.tt.remaining"));
      break;
    case gBTTconstants.DT_UPTIME:
      clock_display.value = calculateBrowserUptime(currenttime);;
      clock_display.setAttribute("tooltiptext", gDbtUiElements.s_bundle.getString("btt.tt.uptime"));
      break;
    default:
      break;
  }

}


/*****************************************************************
*  Snooze - Hitting snooze will bump back alarm time to (current *
*           time + snooze increment) if in ALARM state, or to    *
*           (alarm time + snooze increment) if in TIMING state.  *
*                                                                *
*****************************************************************/

function Snooze()
{
  if (gBTTClockController.alarmState != gBTTconstants.AS_OFF)
   {
     var alarmtime_str;
     var currenttime = new Date();

     if (gBTTClockController.alarmState == gBTTconstants.AS_ALARMING)
      {
        gBTTClockController.targetDate.setTime(currenttime.getTime() + gBTTClockController.snooze_increment * 60 * 1000);
        gBTTClockController.alarmState = gBTTconstants.AS_SET;
        document.getElementById("btt-alarmicon-img").setAttribute("alarming", "false");

        if (gAlarm_TimerID != null)
          clearInterval(gAlarm_TimerID);
      }
     else // timerController.state == gBTTconstants.AS_SET
      {
        gBTTClockController.targetDate.setTime(gBTTClockController.targetDate.getTime() + gBTTClockController.snooze_increment * 60 * 1000);
      }

     alarmtime_str = "Alarm Time: " + dateToClock(gBTTClockController.targetDate);
     document.getElementById("btt-alarmicon-img").setAttribute("tooltiptext", alarmtime_str);
   }
}


function bttShowAlarmInfo()
{
  window.openDialog('chrome://browserticktock/content/dialogAlarmInfo.xul',
                    'bttAlarmInfo','chrome,modal,centerscreen');
}


// Derived, with modifications, from browser.js::loadOneOrMoreURIs()
// and bookmarks.js::openGroupBookmark(). Respects preferences
// browser.tabs.loadFolderAndReplace and browser.tabs.loadBookmarksInBackground

function bttLoadOneOrMoreURIs(aURIString)
{
  // this will be used when BTT is converted to component
  //var w        = getTopWin();
  //var browser  = w.document.getElementById("content");

  var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);

  var bReplace = false;
  var bLoadInBackground = false;
  try {
    prefs.getBoolPref("browser.tabs.loadFolderAndReplace");
    bLoadInBackground = prefs.getBoolPref("browser.tabs.loadBookmarksInBackground");
  } catch(e) { }

  var urls = aURIString.split("|");
  
  var tabPanels = gBrowser.browsers;
  var tabCount  = tabPanels.length;
  var index0;

  // determine starting tab
  if (bReplace)
    index0 = 0;
  else
   {
     for (index0=tabCount-1; index0>=0; index0--)
      if (gBrowser.browsers[index0].webNavigation.currentURI.spec != "about:blank")
        break;
     index0++;
   }
 
  var index  = index0;
  for (var i= 0; i < urls.length; ++i)
   {
     if (index < tabCount)
       tabPanels[index].loadURI(urls[i]);
     else
       gBrowser.addTab(urls[i]);
     index++;
   }
 
  // If the bookmark group was completely invalid, just bail.
  if (index == index0)
    return;

  var tabs = gBrowser.mTabContainer.childNodes;

  // focus the first tab if prefs say to
  if (!bLoadInBackground || bReplace)
   {
     // Select the first tab in the group.
     gBrowser.selectedTab = tabs[index0];
   }
 
  // Close any remaining open tabs that are left over.
  // (Always skipped when we append tabs)
  for (var i = tabCount-1; i >= index; --i)
    gBrowser.removeTab(tabs[i]);

}
 

// Called when BTT toolbar item is dragged onto customize
// toolbar palette

function clearBrowserTickTock()
{
  if (gClock_TimerID != null)
    clearInterval(gClock_TimerID);


  // clear any existing alarm and reset alarm menu item

  gBTTClockController.alarmState = gBTTconstants.AS_OFF;
  document.getElementById("btt-alarm-mitem").setAttribute("checked", "false");

  if (gAlarm_TimerID != null)
    clearInterval(gAlarm_TimerID);


  // clear cuckoo and reset cuckoo menu item

  gBTTClockController.bCuckooOn = false;
  document.getElementById("btt-cuckoo-mitem").setAttribute("checked", "false");

} 

