/****************************************************************************
**  BrowserTickTock - Timer/Clock/Alarm add-on for Mozilla
**  Copyright (c) 2004-2005  Charles Melhorn
**
**  This file is part of BrowserTickTock.
**
**  BrowserTickTock is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  BrowserTickTock is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 

const gBTTPrefConstants =
{
  PT_BOOL:         0,  // preference types
  PT_INT:          1,
  PT_STR:          2,
  PT_UNICHAR:      3,
  PT_UNICHARLOCAL: 4,
  PT_FILE:         5
}

var gBTTPrefService = Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService);

const gBTTUserPreferenceInfo = 
{
  clock_format:       {defvalue: 0, prefname: "clockformat", preftype: gBTTPrefConstants.PT_INT},
  use_clock_color:    {defvalue: false, prefname: "useclockcolor", preftype: gBTTPrefConstants.PT_BOOL},
  clocktext_color:    {defvalue: "", prefname: "clocktextcolor", preftype: gBTTPrefConstants.PT_STR},
  date_format:        {defvalue: "m/d/Y", prefname: "dateformat", preftype: gBTTPrefConstants.PT_STR},
  cuckoo_interval:    {defvalue: 0, prefname: "cuckoointerval", preftype: gBTTPrefConstants.PT_INT},
  cuckoo_playsound:   {defvalue: true, prefname: "cuckooplaysound", preftype: gBTTPrefConstants.PT_BOOL},
  cuckoo_sound_type:  {defvalue: "standard", prefname: "cuckoosoundtype", preftype: gBTTPrefConstants.PT_STR},
  cuckoo_sound_file:  {defvalue: "cc_cuckoo.wav", prefname: "cuckoosoundfile", preftype: gBTTPrefConstants.PT_STR},
  auto_reset:         {defvalue: false, prefname: "autoreset", preftype: gBTTPrefConstants.PT_BOOL},
  alarm_timeout:      {defvalue: 2, prefname: "alarmtimeout", preftype: gBTTPrefConstants.PT_INT},
  alarm_popwindow:    {defvalue: false, prefname: "alarmpopwindow", preftype: gBTTPrefConstants.PT_BOOL},
  alarm_flashbrowser: {defvalue: true, prefname: "alarmflashbrowser", preftype: gBTTPrefConstants.PT_BOOL},
  alarm_showslider:   {defvalue: true, prefname: "alarmshowslider", preftype: gBTTPrefConstants.PT_BOOL},
  alarm_playsound:    {defvalue: true, prefname: "alarmplaysound", preftype: gBTTPrefConstants.PT_BOOL},
  alarm_sound_type:   {defvalue: "standard", prefname: "alarmsoundtype", preftype: gBTTPrefConstants.PT_STR},
  alarm_sound_file:   {defvalue: "alarm_tada.wav", prefname: "alarmsoundfile", preftype: gBTTPrefConstants.PT_STR},
  alarm_loadurl:      {defvalue: false, prefname: "alarmloadurl", preftype: gBTTPrefConstants.PT_BOOL},
  alarm_url:          {defvalue: "", prefname: "alarmurl", preftype: gBTTPrefConstants.PT_STR},
  alarm_preset0:      {defvalue: "", prefname: "alarmpreset0", preftype: gBTTPrefConstants.PT_STR},
  use_fixed_snooze:   {defvalue: false, prefname: "usefixedsnooze", preftype: gBTTPrefConstants.PT_BOOL},
  snooze_increment:   {defvalue: 5, prefname: "snoozeincrement", preftype: gBTTPrefConstants.PT_INT},
  update_interval:    {defvalue: 1, prefname: "updateinterval", preftype: gBTTPrefConstants.PT_INT},
  compute_uptime:     {defvalue: false, prefname: "computeuptime", preftype: gBTTPrefConstants.PT_BOOL},

  prefNode:           gBTTPrefService.getBranch("extensions.btt."),

  getPrefValue:       function(prefId_str)
  {
    return (getBTTUserPreference(this[prefId_str].prefname,
                                this[prefId_str].defvalue,
                                this[prefId_str].preftype,
                                this.prefNode));
  },

  getPrefDefValue:    function(prefId_str)
  {
    return this[prefId_str].defvalue;
  },

  setPrefValue:       function(prefId_str, value)
  {
    setBTTUserPreference(this[prefId_str].prefname,
                        value,
                        this[prefId_str].preftype,
                        this.prefNode);
  }

}


function getBTTUserPreference(prefName, defValue, prefType, prefNode)
{
  var e, rv;

  switch(prefType)
   {
     case gBTTPrefConstants.PT_BOOL:

       try {rv = prefNode.getBoolPref(prefName);}
        catch (e) { rv = defValue; }        

       break;

     case gBTTPrefConstants.PT_INT:
       
       try {rv = prefNode.getIntPref(prefName);}
        catch (e) { rv = defValue; }        

       break;

     case gBTTPrefConstants.PT_STR:

       try {rv = prefNode.getCharPref(prefName);}
        catch (e) { rv = defValue; }        

       break;

     case gBTTPrefConstants.PT_UNICHAR:
     case gBTTPrefConstants.PT_UNICHARLOCAL:
     case gBTTPrefConstants.PT_FILE:

     default:
       alert("Preference type not implemented"); // DEBUG
   }

  return rv;
}


function setBTTUserPreference(prefName, prefValue, prefType, prefNode)
{
  switch(prefType)
   {
     case gBTTPrefConstants.PT_BOOL:
       prefNode.setBoolPref(prefName, prefValue);
       break;

     case gBTTPrefConstants.PT_INT:
       prefNode.setIntPref(prefName, prefValue);
       break;

     case gBTTPrefConstants.PT_STR:
       prefNode.setCharPref(prefName, prefValue);
       break;

     case gBTTPrefConstants.PT_UNICHAR:
     case gBTTPrefConstants.PT_UNICHARLOCAL:
     case gBTTPrefConstants.PT_FILE:

     default:
       alert("Preference type not implemented"); // DEBUG
   }
}


/*****************************************************************
* (code borrowed, with slight modification, from navigator.js - CM)
* Pref listener add/remove functions.
* Both functions assume that observer.domains is set to an array
* of the pref domains we want to start/stop listening to.
*****************************************************************/

function bttAddPrefListener(observer)
{
  try {
    var pbi = gBTTPrefService.QueryInterface(Components.interfaces.nsIPrefBranchInternal);

    for (var i = 0; i < observer.domains.length; i++)
      pbi.addObserver(observer.domains[i], observer, false);

  } catch(ex) {
    alert("Failed to add prefs: " + ex + "\n");
  }
}
 
function bttRemovePrefListener(observer)
{
  try {
    var pbi = gBTTPrefService.QueryInterface(Components.interfaces.nsIPrefBranchInternal);

    for (i=0; i < observer.domains.length; i++)
       pbi.removeObserver(observer.domains[i], observer);

  } catch(ex) {
    alert("Failed to remove pref observer: " + ex + "\n");
  }
}

