function xMoveButton () {
}

xMoveButton.prototype = {

  Cc: Components.classes,
  Ci: Components.interfaces,

  _extension: null,
  _buttonID: null,
  _windowID: null,
  _prefBranch: null,
  _prefButtonBar: null,
  _prefButtonPos: null,
  _defaultBar: null,
  _defaultPos: null,
  _configURL: null,
  _doc: null,
  _dialogDoc: null,

  _barListID: null,
  _barPopupID: null,
  _radioGroupID: null,
  _radioGroup: null,
  _textPosID: null,
  _validPosID: null,

  mPositionMax: null,

  prefs: Components.classes["@mozilla.org/preferences-service;1"]
            .getService(Components.interfaces.nsIPrefService)
            .QueryInterface(Components.interfaces.nsIPrefBranch2)
            .QueryInterface(Components.interfaces.nsIPrefBranch),

  getPref: function xMoveButtonGetPref(prefName, defaultValue, isComplex) {
    try {
      switch (this.prefs.getPrefType(prefName)) {
        case this.prefs.PREF_BOOL:
          return this.prefs.getBoolPref(prefName);
        case this.prefs.PREF_INT:
          return this.prefs.getIntPref(prefName);
        case this.prefs.PREF_STRING:
          if (isComplex)
            return this.prefs.getComplexValue(prefName, Components.interfaces.nsIPrefLocalizedString).data;
          else
            return this.prefs.getCharPref(prefName);
        case this.prefs.PREF_STRING:
          return this.prefs.getCharPref(prefName);
        }
    }
    catch (ex) { /* return the default value */ }
    
    return defaultValue;
  },

  prefObserver : {
    outer: null,
    observe: function(subject, topic, data) {
      if(topic == "nsPref:changed") {
        this.outer.moveButton();
      }
    },

    QueryInterface : function (aIID) {
      if (aIID.equals(Components.interfaces.nsIObserver) || 
      aIID.equals(Components.interfaces.nsISupports) ||
      aIID.equals(Components.interfaces.nsISupportsWeakReference))
        return this;
      throw Components.results.NS_NOINTERFACE;
    }
  },

  removePrefObserver : function xMoveButtonRemovePrefObserver() {
    this.prefs.removeObserver(this._prefBranch, this.prefObserver);
  },

/*
  Method: initButton
  Arguments:
    extension  // Extension name or something to make this button unique.
    buttonID,  // ID of button to move.
    defaultBar // Initial default bar location.
    defaultPos // Initial default position in bar; -1 means always last.
    configURL  // xMovebutton dialog chrome URL.
*/
  initButton: function xMoveButtonInit( extension, buttonID, defaultBar, defaultPos, configURL ) {

    this.Cc = Components.classes;
    this.Ci = Components.interfaces;
    this.prefs = this.Cc["@mozilla.org/preferences-service;1"]
            .getService(this.Ci.nsIPrefService)
            .QueryInterface(this.Ci.nsIPrefBranch2)
            .QueryInterface(this.Ci.nsIPrefBranch);

    this._extension = extension;
    this._buttonID = buttonID;
    this._defaultBar = defaultBar ? defaultBar : "status-bar";
    this._defaultPos = defaultPos ? defaultPos : -1;
    this._configURL = configURL ? configURL : "chrome:xMoveButton.xul";
    var wintype = document.documentElement.getAttribute("windowtype");
    var winID = document.documentElement.getAttribute("id");
    this._windowID = wintype ? wintype : "general_" + winID;
    this._prefBranch = "extensions." + this._extension + "." +
      this._buttonID + "." + this._windowID + ".";
    this._prefButtonBar = this._prefBranch + "location";
    this._prefButtonPos = this._prefBranch + "position";

    this.moveButton();
    this.onload();
    //window.addEventListener("unload", this.onunload, false);
  },

  onload: function xMoveButtonOnLoad() {
    this.prefObserver.outer = this;
    this.prefs.addObserver(this._prefBranch, this.prefObserver, true);
  },

  onunload: function xMoveButtonOnUnload() {
    //this.removePrefObserver();
    this.prefs.removeObserver(this._prefBranch, this.prefObserver);
  },

  moveButton: function xMoveButtonMoveButton() {
    //check if box is in the right location
    var toolbar = document.getElementById(
      this.getPref(this._prefButtonBar, null));
    var position = this.getPref(this._prefButtonPos, this._defaultPos);

    var box = document.getElementById(this._buttonID);

    //if toolbar doesn't exist move to default location
    if (!toolbar) {
      this.prefs.setCharPref(this._prefButtonBar, this._defaultBar);
      this.prefs.setIntPref(this._prefButtonPos, this._defaultPos);
      return;
    };

    // bail if it is in the right place
    if (this.indexOf(toolbar, box) == position)
      return;

    //remove us from parent
    box.parentNode.removeChild(box);
    
    //make sure we have the right element type
    var newbox = null;
    if (toolbar.localName == "statusbar") {
      if (box.localName != "statusbarpanel")
        newbox = document.createElement("statusbarpanel");
    } else {
      if (box.localName == "statusbarpanel")
        newbox = document.createElement("toolbaritem");    
    };
    
    //append children of old box
    if (newbox) {
      newbox.setAttribute("id", this._buttonID);
      var attrs = box.attributes;
      //for(var i = attrs.length - 1; i >= 0; i--) {
      for(var i = 0; i < attrs.length; i++) {
        newbox.setAttribute(attrs[i].name, attrs[i].value);
      }
      while (box.hasChildNodes())
        newbox.appendChild(box.firstChild);      
    } else
      newbox = box;
    
    //insert us in correct place
    this.insertAtIndex(toolbar, newbox, position);
  },

  indexOf: function xMoveButtonIndexOf(aParent, aChild) {
    // -1 if it does not exist
    var children = aParent.childNodes;
    for (var x=0; x<children.length; x++)
      if (children[x] == aChild)
        return (x == children.length - 1) ? -1 : x;
   
    return null;
  },
  
  insertAtIndex: function xMoveButtonInsertAtIndex(aParent, aChild, aIndex) {
    var children = aParent.childNodes;
    if ((children.length == 0) || (aIndex >= children.length) || (aIndex < 0))
      aParent.appendChild(aChild);
    else
      aParent.insertBefore(aChild, children[aIndex]);
  },

  openConfig: function xMoveButtonOpenConfig() {
    this._doc = document;
    openDialog( this._configURL, "", "chrome,all,resizable=yes,dependent=yes",
      this);
  },

  initDialog: function xMoveButtonInitDialog() {
    this.populateBars();
    this.populatePositions();
  },

  concat: function xMoveButtonConcat(c1, c2) {
    // Concats two collections into an array.
    var c3 = new Array(c1.length + c2.length);
    var x,y = 0;

    for (x = 0; x < c1.length; x++)
      c3[y++] = c1[x];

    for (x = 0; x < c2.length; x++)
      c3[y++] = c2[x];

    return c3;
  },

  populateBars: function xMoveButtonPopulateBars() {
    // Creates the menuitems for the toolbar selector.
    var doc = this._doc;
    var ddoc = this._dialogDoc;
    var toolbars = doc.getElementsByTagName("toolbar");
    var statusbars = doc.getElementsByTagName("statusbar");
    var menubars = doc.getElementsByTagName("menubar");
    var popup = ddoc.getElementById(this._barPopupID);
    var x, bar, item, val, list;
    
    // first remove the toolbars already there...
    while (popup.hasChildNodes())
      popup.removeChild(popup.firstChild);

    toolbars = this.concat(menubars, toolbars);
    toolbars = this.concat(toolbars, statusbars);
    for (x=0; x<toolbars.length; x++) {
      var bar = toolbars[x];
      
      //do not include find toolbar
      if (!bar.hasAttribute("id") ||
          bar.getAttribute("id") == "FindToolbar")
        continue;
             
      item = ddoc.createElement("menuitem");
      item.setAttribute("id", bar.getAttribute("id"));
      item.value = bar.getAttribute("id");         
      if (bar.hasAttribute("toolbarname"))
        item.setAttribute("label", bar.getAttribute("toolbarname"));
      else
        item.setAttribute("label", bar.getAttribute("id"));                 
      popup.appendChild(item);
    }
    
    this.setElement(this._barListID, this._prefButtonBar, "Char", this._barPopupID);    
  },

  populatePositions: function xMoveButtonPopulatePositions() {
    if (!this._dialogDoc.getElementById(this._barListID).selectedItem) {
      this.prefs.setCharPref(_prefButtonBar, _defaultBar);
      this.prefs.setIntPref(_prefButtonPos, _defaultPos);
      this.setElement(this._barListID, _prefButtonBar, "Char", _barPopupID);
    }

    var barid = this._dialogDoc.getElementById(this._barListID).selectedItem.getAttribute("id");
    var bar = this._doc.getElementById(barid);
    if (!bar) {
      this.mPositionMax = 0;
    }
    else {
      var len = bar.childNodes.length;
      //don't include ourself in count
      if (this._doc.getElementById(this._buttonID) &&
          this._doc.getElementById(this._buttonID).parentNode == bar)
        len--;
      this.mPositionMax = len;
    }

    var val = this.prefs.getIntPref(this._prefButtonPos);
    var val2 = this.prefs.getCharPref(this._prefButtonBar);
    var text = this._dialogDoc.getElementById(this._textPosID);
    var radio = this._dialogDoc.getElementsByAttribute("group", "position");
    var group = this._dialogDoc.getElementById(this._radioGroupID);
    var x, el;
    //set max value
    text.value = this.mPositionMax;
    //mark always last
    if (val == -1) {
      for (x=0; x < radio.length; x++) {
        if (parseInt(radio[x].value) == val) {
          group.selectedItem = radio[x];
          break;
        }
      }
    }
    //mark specific position
    else {
      for (x=0; x < radio.length; x++) {
        if (parseInt(radio[x].value) != -1) {
          group.selectedItem = radio[x];
          break;
        }
      }    
      if (barid != val2) {
        text.value = this.mPositionMax;
      } else {
        if (val < 0)
          text.value = this.mPositionMax;
        else if (val > this.mPositionMax)
          text.value = 0;
        else
          text.value = val; 
      }
    }
    // update the error label
    el = this._dialogDoc.getElementById(this._validPosID);
    el.setAttribute("value", "0-"+this.mPositionMax);
    //revalidate position
    //setTimeout("this.validate()", 10);
  },

  setElement: function xMoveButtonSetElement(aName, aPref, aType, aGroup) {
    var x, el, els, val;
    
    el = this._dialogDoc.getElementById(aName);
    val = this.getPref(aPref, null, aType == "Complex");
    switch (el.localName) {
      case "checkbox":
        el.checked = val;
        break;
      case "textbox":
        el.value = val;
        break;
      case "menulist":
        els = this._dialogDoc.getElementById(aGroup).childNodes;        
        for (x=0; x < els.length; x++) {
          if (((aType == "Int") ? parseInt(els[x].value) : els[x].value) == val) {
            el.selectedItem = els[x];
            break;
          }
        }      
        break;            
      case "radiogroup":
        els = this._dialogDoc.getElementsByAttribute("group", aGroup);        
        for (x=0; x < els.length; x++) {
          if (((aType == "Int") ? parseInt(els[x].value) : els[x].value) == val) {
            el.selectedItem = els[x];
            break;
          }
        }      
        break;
    }  
  },

  checkPosition: function xMoveButtonCheckPosition() {
    var rv = false;
    var el = this._dialogDoc.getElementById(this._textPosID);
    var el2 = this._dialogDoc.getElementById(this._validPosID);
    if (isNaN(parseInt(el.value)) || el.value < 0 || el.value > this.mPositionMax) {
      el2.setAttribute("value", "0-"+this.mPositionMax+" !!!!");
      el.focus();
    }
    else {
      el.setAttribute("value", "0-"+this.mPositionMax);
      rv = true;
    }
    return rv;
  },

  setOption: function xMoveButtonSetOption(aName, aPref, aType)
  {
    var el = this._dialogDoc.getElementById(aName);
    var val;
    switch (el.localName) {
      case "checkbox":
        val = el.checked;
        break;
      case "textbox":
        val = (aType == "Int") ? parseInt(el.value) : el.value;
        break;
      case "menulist":
      case "radiogroup":
        val = (aType == "Int") ? parseInt(el.selectedItem.value) : el.selectedItem.value;
        break;
    };
    
    switch (aType) {
      case "Complex":
        var support = Components.classes["@mozilla.org/supports-string;1"]
                      .createInstance(Components.interfaces.nsISupportsString);  
        support.data = val;
        this.prefs.setComplexValue(aPref, Components.interfaces.nsISupportsString, support);  
        break;
      case "Bool":
        this.prefs.setBoolPref(aPref, val);
        break;
      case "Int":
        this.prefs.setIntPref(aPref, val);
        break;
      case "Char":
        this.prefs.setCharPref(aPref, val);
        break;
      default:
        break;   
    };
  },

  saveOptions: function xMoveButtonSaveOptions() {
    this.setOption(this._barListID, this._prefButtonBar, "Char");
    var position = this._dialogDoc.getElementById(this._radioGroupID);
    if (parseInt(position.selectedItem.value) == -1)
      this.prefs.setIntPref(this._prefButtonPos, -1);
    else
      this.setOption(this._textPosID, this._prefButtonPos, "Int");
    this.moveButton();
  },

  dialogApply: function xMoveButtonDialogApply() {
    if (this.checkPosition());
      this.saveOptions();
    return false;
  },

  dialogAccept: function xMoveButtonDialogAccept() {
    var rv = this.checkPosition();
    if (rv);
      this.saveOptions();
    return rv;
  },

  isSM: function xMoveButtonIsSM() {
    const SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}"; 
    var nsIXULAppInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                                  .getService(Components.interfaces.nsIXULAppInfo);
    return (nsIXULAppInfo.ID == SEAMONKEY_ID);
  }

}